from objc import YES, NO, nil
from AppKit import NSApplication, NSImage, NSBitmapImageRep, NSPasteboard, NSGraphicsContext
import AppKit

import getopt, os.path, sys

# **************************************************************************************************

NSApp = NSApplication.sharedApplication()

# **************************************************************************************************

def sliceImage(imagePath, sliceRect, outputSize):
    className  = os.path.splitext(os.path.basename(imagePath))[0]

    image = NSImage.alloc().initWithContentsOfFile_(imagePath)
    image.setFlipped_(YES)
    size = [int(coord) for coord in image.size()]
    
    imageName, imageExt = os.path.splitext(imagePath)
    bordersPath = "%sBorders%s" % (imageName, imageExt)
    cornersPath = "%sCorners%s" % (imageName, imageExt)

    stretchedImage = getStretchedImage(image, sliceRect, size, outputSize)
    saveImage(stretchedImage, cornersPath)

    vborderImage = getVerticalBorderImage(image, sliceRect, size, outputSize)
    saveImage(vborderImage, bordersPath)
    
    writeCSS(bordersPath, cornersPath, className, sliceRect, size, outputSize)

def writeCSS(bordersPath, cornersPath, className, sliceRect, inputSize, outputSize):
    leftWidth = sliceRect[0]
    rightWidth = inputSize[0] - sliceRect[2]

    cssFile = file("%s.css" % className, "w")
    
    cssFile.write(
        ".%sInner {\n" % className +
        "    padding: 1px;\n" +
        "}\n\n" +
        
        ".%sInner1 {\n" % className +
        "    padding-left: %spx;\n" % leftWidth +
        "    background: url(%s) repeat-y;\n" % bordersPath +
        "}\n\n" +

        ".%sInner2 {\n" % className +
        "    padding-right: %spx;\n" % rightWidth +
        "    background: url(%s) repeat-y 100%% 0;\n" % bordersPath +
        "}\n\n" +

        ".%sTop1 {\n" % className +
        "    background: url(%s) no-repeat 100%% 0;\n" % cornersPath +
        "    margin-left: %spx;\n" % sliceRect[0] +
        "    height: %spx;\n" % sliceRect[1] +
        "}\n\n" +

        ".%sTop2 {\n" % className +
        "    position: relative;\n" +
        "    left: %spx;\n" % (-1 * sliceRect[0]) +
        "    width: %spx;\n" % sliceRect[0] +
        "    height: %spx;\n" % sliceRect[1] +
        "    background: url(%s) no-repeat;\n" % cornersPath +
        "}\n\n" +

        ".%sBottom1 {\n" % className +
        "    background: url(%s) no-repeat 100%% 100%%;\n" % cornersPath +
        "    margin-left: %spx;\n" % sliceRect[0] +
        "    height: %spx;\n" % (inputSize[1] - sliceRect[3]) +
        "}\n\n" +

        ".%sBottom2 {\n" % className +
        "    position: relative;" +
        "    left: %spx;\n" % (-1 * sliceRect[0]) +
        "    width: %spx;\n" % sliceRect[0] +
        "    height: %spx;\n" % (inputSize[1] - sliceRect[3]) +
        "    background: url(%s) no-repeat 0 100%%;\n" % cornersPath +
        "}\n"
    )
    
    cssFile.close()
    
def getStretchedImage(image, sliceRect, inputSize, outputSize):
    newImage = NSImage.alloc().initWithSize_(outputSize)
    newImage.setFlipped_(YES)
    
    innerRight = outputSize[0] - (inputSize[0] - sliceRect[2])
    innerBottom = outputSize[1] - (inputSize[1] - sliceRect[3])
    
    newImage.lockFocus()

    # Top Left
    drawImageSlice(image,
        (0, 0, sliceRect[0], sliceRect[1]),
        (0, 0, sliceRect[0], sliceRect[1]))

    # Top
    drawImageSlice(image,
        (sliceRect[0], 0, sliceRect[0]+1, sliceRect[1]),
        (sliceRect[0], 0, innerRight, sliceRect[1]))

    # Top Right
    drawImageSlice(image,
        (sliceRect[2], 0, inputSize[0], sliceRect[1]),
        (innerRight, 0, outputSize[0], sliceRect[1]))

    # Left
    drawImageSlice(image,
        (0, sliceRect[1], sliceRect[0], sliceRect[1]+1),
        (0, sliceRect[1], sliceRect[0], innerBottom))

    # Bottom Left
    drawImageSlice(image,
        (0, sliceRect[3], sliceRect[0], inputSize[1]),
        (0, innerBottom, sliceRect[0], outputSize[1]))

    # Bottom
    drawImageSlice(image,
        (sliceRect[0], sliceRect[3], sliceRect[0]+1, inputSize[1]),
        (sliceRect[0], innerBottom, innerRight, outputSize[1]))

    # Bottom Right
    drawImageSlice(image,
        (sliceRect[2], sliceRect[3], inputSize[0], inputSize[1]),
        (innerRight, innerBottom, outputSize[0], outputSize[1]))

    # Right
    drawImageSlice(image,
        (sliceRect[2], sliceRect[1], inputSize[0], sliceRect[1]+1),
        (innerRight, sliceRect[1], outputSize[0], innerBottom))

    newImage.unlockFocus()
        
    return newImage

def getVerticalBorderImage(image, sliceRect, inputSize, outputSize):
    leftWidth = sliceRect[0]
    rightWidth = inputSize[0] - sliceRect[2]
    outputHeight = 1
    
    newImage = NSImage.alloc().initWithSize_((leftWidth+rightWidth, outputHeight))
    newImage.setFlipped_(YES)
        
    newImage.lockFocus()

    # Left
    drawImageSlice(image,
        (0, sliceRect[1], sliceRect[0], sliceRect[1]+1),
        (0, 0, sliceRect[0], outputHeight))

    # Right
    drawImageSlice(image,
        (sliceRect[2], sliceRect[1], inputSize[0], sliceRect[1]+1),
        (sliceRect[0], 0, sliceRect[0]+rightWidth, outputHeight))

    newImage.unlockFocus()
        
    return newImage

# **************************************************************************************************

def drawImageSlice(image, srcRect, destRect):
    
    srcWidth = srcRect[2] - srcRect[0]
    srcHeight = srcRect[3] - srcRect[1]
    srcNSRect = ((srcRect[0], srcRect[1]), (srcWidth, srcHeight))

    destWidth = destRect[2] - destRect[0]
    destHeight = destRect[3] - destRect[1]
    destNSRect = ((destRect[0], destRect[1]), (destWidth, destHeight))

    image.drawInRect_fromRect_operation_fraction_(destNSRect, srcNSRect,
        AppKit.NSCompositeSourceOver, 1.0)

def saveImage(image, path):
    """Writes an NSImage to disk as a gif, png, jpg, or tiff."""
    
    tiffData = image.TIFFRepresentation()
    bitmap = NSBitmapImageRep.alloc().initWithData_(tiffData)
    
    fileType = getImageFileType(path)
    if fileType:
        imageData = bitmap.representationUsingType_properties_(fileType, nil)
        imageData.writeToFile_atomically_(path, YES)

def getImageFileType(imagePath):
    ext = os.path.splitext(imagePath)[1].lower()
    
    if ext == ".png":
        return AppKit.NSPNGFileType
    elif ext in (".jpg", ".jpeg"):
        return AppKit.NSJPEGFileType
    elif ext == ".gif":
        return AppKit.NSGIFFileType
    elif ext == ".tiff":
        return AppKit.NSTIFFFileType

# **************************************************************************************************

def main():
    if len(sys.argv) < 4:
        sys.stderr.write("usage: roundme.py imagePath --rect 'x1 y1 x2 y2' [--size 'width height']\n")
        return

    imagePath = sys.argv[1]
    rect = []
    size = []

    opts, args = getopt.getopt(sys.argv[2:], "r:w:", ["rect=","size="])
    
    for name,val in opts:
        if name == "-r" or name == "--rect":
            rect = [int(coord) for coord in val.split(" ")]
        elif name == "-w" or name == "--size":
            size = [int(coord) for coord in val.split(" ")]
    
    if not os.path.isfile(imagePath):
        sys.stderr.write("%s is not a valid image file\n" % imagePath)
        return        

    if len(rect) < 4:
        sys.stderr.write("--rect must contain exactly four coordinates\n");
        return

    if len(size) < 2:
        sys.stderr.write("--size must contain exactly two dimensions\n");
        return
        
    sliceImage(imagePath, rect, size)

if __name__ == "__main__":
    main()
